"""empty message

Revision ID: 4aa8e76fb21d
Revises: 7bb51084235d
Create Date: 2022-09-08 15:16:32.855071

"""
from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
import sqlmodel


# revision identifiers, used by Alembic.
revision = '4aa8e76fb21d'
down_revision = '7bb51084235d'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
